/*!
*
*    RaphaëlTools : Raphaël extension (button, sliders, misc..)
*                     version 0.6.1 (2012-05-11)
*
*            Copyright (C)  2011 - 2012  Jan Stransky
*
*      Czech Technical University, Faculty of Civil Engineering,
*  Department of Structural Mechanics, 166 29 Prague, Czech Republic
*
*
*  RaphaëlTools is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  RaphaëlTools is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @fileOverview <a href="http://raphaeljs.com/">Raphaël</a> (svg javascript library) extension by various functions and "classes" (slider, button, etc.). Requires <a href="http://raphaeljs.com/">raphael.js</a> loaded before loading this file. For more information see <a href="http://mech.fsv.cvut.cz/~stransky/software/raphaeltools/">project homepage</a>.
<br /><br/>RaphaëlTools is a free software distributed under <a href='http://www.gnu.org/licenses/gpl.html'>GNU GPL license</a>.
* @author <a href="http://mech.fsv.cvut.cz/~stransky/">Jan Stránský</a>
* @version Version 0.6.1 (2012-05-11)
*/

/** Generic function used for draging ojects. For each dragable object should be defined function onDragUpdate(dx,dy): e.g. moving only in x direction etc.
* @param {float} dx x increment of dragging
* @param {float} dy y increment of dragging
* @example
* c = Circle.create(100,59,20);
* c.onDragUpdate = function(dx,dy) { c.translate(dx,dy); }
* c.drag(onDragMove,onDragStart,onDragStop) // make an object draggable using c.onDragUpdate() function when moving
*/
function onDragMove(dx,dy) {
	var unit = Raphael.raphaeltoolsDrawingUnit || 1.0;
	this.onDragUpdate(1/unit*(dx - (this.deltax || 0)), 1/unit*(dy - (this.deltay || 0)));
	this.deltax = dx;
	this.deltay = dy;
}

/** Generic function used for draging ojects, see {@link onDragMove}
*/
function onDragStart() { this.deltax = this.deltay = 0; }

/** Generic function used for draging ojects, see {@link onDragMove}
*/
function onDragStop() { this.onDragStop(); }


/* ***********************************************************
* other generic functions
************************************************************/
/** Raphael function loaded from <a href="http://raphaeljs.com">raphael.js file</a>
* @class
*/
Raphael = Raphael;

/** Raphael.el element of {@link Raphael}
* @memberOf Raphael#
*/
Raphael.el = Raphael.el;

/**#@+
* @function
* @memberOf Raphael#
*/
// color setting
/** Sets fill color of receiver to red*/     Raphael.el.fillRed     = function() { return this.attr({"fill":"#f00"}); }
/** Sets fill color of receiver to green*/   Raphael.el.fillGreen   = function() { return this.attr({"fill":"#0f0"}); }
/** Sets fill color of receiver to blue*/    Raphael.el.fillBlue    = function() { return this.attr({"fill":"#00f"}); }
/** Sets fill color of receiver to yellow*/  Raphael.el.fillYellow  = function() { return this.attr({"fill":"#ff0"}); }
/** Sets fill color of receiver to cyan*/    Raphael.el.fillCyan    = function() { return this.attr({"fill":"#0ff"}); }
/** Sets fill color of receiver to magenta*/ Raphael.el.fillMagenta = function() { return this.attr({"fill":"#f0f"}); }
/** Sets fill color of receiver to orange*/  Raphael.el.fillOrange  = function() { return this.attr({"fill":"#f50"}); }
/** Sets fill color of receiver to black*/   Raphael.el.fillBlack   = function() { return this.attr({"fill":"#000"}); }
/** Sets fill color of receiver to white*/   Raphael.el.fillWhite   = function() { return this.attr({"fill":"#fff"}); }
/** Sets fill color of receiver to gray*/    Raphael.el.fillGray    = function() { return this.attr({"fill":"#777"}); }
/** Sets stroke color of receiver to red*/     Raphael.el.strokeRed     = function() { return this.attr({"stroke":"#f00"}); }
/** Sets stroke color of receiver to green*/   Raphael.el.strokeGreen   = function() { return this.attr({"stroke":"#0f0"}); }
/** Sets stroke color of receiver to blue*/    Raphael.el.strokeBlue    = function() { return this.attr({"stroke":"#00f"}); }
/** Sets stroke color of receiver to yellow*/  Raphael.el.strokeYellow  = function() { return this.attr({"stroke":"#ff0"}); }
/** Sets stroke color of receiver to cyan*/    Raphael.el.strokeCyan    = function() { return this.attr({"stroke":"#0ff"}); }
/** Sets stroke color of receiver to magenta*/ Raphael.el.strokeMagenta = function() { return this.attr({"stroke":"#f0f"}); }
/** Sets stroke color of receiver to orange*/  Raphael.el.strokeOrange  = function() { return this.attr({"stroke":"#f50"}); }
/** Sets stroke color of receiver to black*/   Raphael.el.strokeBlack   = function() { return this.attr({"stroke":"#000"}); }
/** Sets stroke color of receiver to white*/   Raphael.el.strokeWhite   = function() { return this.attr({"stroke":"#fff"}); }
/** Sets stroke color of receiver to gray*/    Raphael.el.strokeGray    = function() { return this.attr({"stroke":"#777"}); }
/** Sets fill and stroke color of receiver to red*/     Raphael.el.red     = function() { this.fillRed();     return this.strokeRed(); }
/** Sets fill and stroke color of receiver to green*/   Raphael.el.green   = function() { this.fillGreen();   return this.strokeGreen(); }
/** Sets fill and stroke color of receiver to blue*/    Raphael.el.blue    = function() { this.fillBlue();    return this.strokeBlue(); }
/** Sets fill and stroke color of receiver to yellow*/  Raphael.el.yellow  = function() { this.fillYellow();  return this.strokeYellow(); }
/** Sets fill and stroke color of receiver to cyan*/    Raphael.el.cyan    = function() { this.fillCyan();    return this.strokeCyan(); }
/** Sets fill and stroke color of receiver to magenta*/ Raphael.el.magenta = function() { this.fillMagenta(); return this.strokeMagenta(); }
/** Sets fill and stroke color of receiver to orange*/  Raphael.el.orange  = function() { this.fillOrange();  return this.strokeOrange(); }
/** Sets fill and stroke color of receiver to black*/   Raphael.el.black   = function() { this.fillBlack();   return this.strokeBlack(); }
/** Sets fill and stroke color of receiver to white*/   Raphael.el.white   = function() { this.fillWhite();   return this.strokeWhite(); }
/** Sets fill and stroke color of receiver to gray*/    Raphael.el.gray    = function() { this.fillGreen();   return this.strokeGray(); }
/** Set width of receiver
* @param w new width
*/
Raphael.el.setWidth = function(w) { return this.attr({"stroke-width":w}); }
/**#@-*/

/** Returns [x,y] components of transformated vector
* @param {float} x x component of original vector
* @param {float} y y component of original vector
* @param {float} sin sin of angle of rotation
* @param {float} cos cos of angle of rotation
* @returns {[float,float]} coordinates of transformated vector
*/
vecTrans = function(x,y,sin,cos) { return [x*cos-y*sin,x*sin+y*cos]; }







/** 2d xy plane point implementation
* @class represents points in 2d xy plane
* @param {float} x x coordinate of point
* @param {float} y y coordinate of point
* @property {float} x x coordinate
* @property {float} y y coordinate
* @property {[Points2d]} copies array of copies of itself
* @property {[[float,float]]} copiesShifts horizontal and verical shifts of copies from receiver
*/
Point2d = function(x,y) {
	this.x = x || 0.;
	this.y = y || 0.;
	this.copies = [];
	this.copiesShifts = []
}

/** String representation
* @returns {String} string representation
*/
Point2d.prototype.toString = function() {
	return "Point2d ( "+this.x+","+this.y+" )";
}

/** Copy receiver to given positions
* @param {[[float,float]]} shifts vertical and horizontal shifts of copied points
* @example n = Point2d(100,150);
* shifts=[[100,200],[200,200],[100,300],[200,300]]
*/
Point2d.prototype.copy = function(shifts) {
	for (var i=0; i<shifts.length; i++) {
		this.copiesShifts.push(shifts[i]);
		this.copies.push(Point2d.create(this.x+shifts[i][0],this.y+shifts[i][1]));
	}
}

/** Sets new position to receiver
* @param {float} x x coordinate of new position
* @param {float} y y coordinate of new position
*/
Point2d.prototype.setXY = function(x,y) {
	this.x=x;
	this.y=y;
	this.update();
}

/** Set new x coordinate to receiver
* @param x x coordinate of new position
*/
Point2d.prototype.setX = function(x) {
	this.x=x;
	this.update();
}

/** Set new y coordinate to receiver
* @param y y coordinate of new position
*/
Point2d.prototype.setY = function(y) {
	this.y=y;
	this.update();
}

/** Change position of receiver by given values
* @param {float} dx length of translation in x direction
* @param {float} dy length of translation in y direction
*/
Point2d.prototype.translate = function(dx,dy) {
	this.x+=dx;
	this.y+=dy;
	this.update();
}

/** Change horizontal position of receiver by given values
* @param {float} dx length of translation in x direction
*/
Point2d.prototype.translateX = function(dx) {
	this.translate(dx,0.);
}

/** Change vertical position of receiver by given values
* @param {float} dy length of translation in x direction
*/
Point2d.prototype.translateY = function(dy) {
	this.translate(0.,dy);
}

/** Update receiver (change positions of stored copies so as stored shifts between receiver and its copies remain constant
*/
Point2d.prototype.update = function() {
	for (var i=0; i<this.copies.length; i++) {
		this.copies[i].setXY(this.x+this.copiesShifts[i][0],this.y+this.copiesShifts[i][1]);
		this.copies[i].update();
	}
}

/** Creates new Point2d object, for parameters meaning see {@link Point2d}
* @returns {Point2d} new Point2d object
*/
Point2d.create = function(x,y) {
	var ret = new Point2d(x,y);
	ret.update();
	return ret;
}




/** 3d point implementation
* @class represents points in 3d space
* @param {float} xr x coordinate of point in real space
* @param {float} yr y coordinate of point in real space
* @param {float} zr z coordinate of point in real space
* @property {float} xr x coordinate of point in real space
* @property {float} yr y coordinate of point in real space
* @property {float} zr z coordinate of point in real space
* @property {float} x x coordinate of point in screen projection
* @property {float} y y coordinate of point in screen projection (can be used for visibility issues)
* @property {float} z z coordinate of point in screen projection
* @property {[3x(3 or 4) floats]} p projection matrix
* @property {[Points]} copies array of copies of itself
* @property {[[float,float]]} copiesShifts horizontal and verical shifts of copies from receiver
*/
Point3d = function(xr,yr,zr,projection) {
	this.xr = xr || 0;
	this.yr = yr || 0;
	this.zr = zr || 0;
	this.copies = [];
	this.copiesShifts = []
	this.p = projection==undefined? [[1,0,0],[0,1,0],[0,0,1]] : projection;
	this.x = this.p[0][0]*this.xr + this.p[0][1]*this.yr + this.p[0][2]*this.zr + (this.p[0][3] || 0);
	this.y = this.p[1][0]*this.xr + this.p[1][1]*this.yr + this.p[1][2]*this.zr + (this.p[1][3] || 0);
	this.z = this.p[2][0]*this.xr + this.p[2][1]*this.yr + this.p[2][2]*this.zr + (this.p[2][3] || 0);
}

/** String representation
* @returns {String} string representation
*/
Point3d.prototype.toString = function() {
	return "Point3d ( "+this.xr+","+this.yr+","+this.zr+" )";
}

/** Copy receiver to given positions
* @param {[[float,float,float]]} shifts vertical and horizontal shifts of copied points
* @example n = Point3d(100,150,250);
* shifts=[[100,200,300],[200,200,500],[100,300,100],[200,300,340]]
*/
Point3d.prototype.copy = function(shifts) {
	for (var i=0; i<shifts.length; i++) {
		this.copiesShifts.push(shifts[i]);
		this.copies.push(new Point3d(this.xr+shifts[i][0],this.yr+shifts[i][1],this.zr+shifts[i][2]));
	}
}

/** Sets new position to receiver
* @param {float} xr xr coordinate of new position (in real space)
* @param {float} yr yr coordinate of new position (in real space)
* @param {float} zr zr coordinate of new position (in real space)
*/
Point3d.prototype.setXYZ = function(xr,yr,zr) {
	this.xr=xr;
	this.yr=yr;
	this.zr=zr;
	this.update();
}

/** Set new x coordinate to receiver
* @param x x coordinate of new position (in real space)
*/
Point3d.prototype.setX = function(xr) {
	this.xr=xr;
	this.update();
}

/** Set new y coordinate to receiver
* @param y y coordinate of new position (in real space)
*/
Point3d.prototype.setY = function(yr) {
	this.yr=yr;
	this.update();
}

/** Set new z coordinate to receiver
* @param z z coordinate of new position (in real space)
*/
Point3d.prototype.setZ = function(zr) {
	this.zr=zr;
	this.update();
}

/** Change position of receiver by given values
* @param {float} dx length of translation in x direction (in real space)
* @param {float} dy length of translation in y direction (in real space)
* @param {float} dz length of translation in z direction (in real space)
*/
Point3d.prototype.translate = function(dx,dy,dz) {
	this.xr+=dx;
	this.zr+=dz;
	this.yr+=dy;
	this.update();
}

/** Change horizontal position of receiver by given values
* @param {float} dx length of translation in x direction (in real space)
*/
Point3d.prototype.translateX = function(dx) {
	this.translate(dx,0.,0.);
}

/** Change horizontal position of receiver by given values
* @param {float} dy length of translation in y direction (in real space)
*/
Point3d.prototype.translateY = function(dy) {
	this.translate(0.,dy,0.);
}

/** Change vertical position of receiver by given values
* @param {float} dz length of translation in x direction (in real space)
*/
Point3d.prototype.translateZ = function(dz) {
	this.translate(0.,0.,dz);
}

/** Sets projection matrix
* @param {[2x(3 or 4) floats]} p new projection matrix
*/
Point3d.prototype.setProjection = function(p) {
	this.p = p;
	this.update()
}

/** project receiver to xz screen coordinates accordind to this.p
*/
Point3d.prototype.project = function() {
	this.x = this.p[0][0]*this.xr + this.p[0][1]*this.yr + this.p[0][2]*this.zr + (this.p[0][3] || 0);
	this.y = this.p[1][0]*this.xr + this.p[1][1]*this.yr + this.p[1][2]*this.zr + (this.p[1][3] || 0);
	this.z = this.p[2][0]*this.xr + this.p[2][1]*this.yr + this.p[2][2]*this.zr + (this.p[2][3] || 0);
}

/** Rotate receiver around x axis of angle a
* @param {float} a angle of rotation
*/
Point3d.prototype.rotX = function(a) {
	c = Math.cos(a);
	s = Math.sin(a);
	var p = this.p;
	var p10 = p[1][0], p11 = p[1][1], p12 = p[1][2];
	var p20 = p[2][0], p21 = p[2][1], p22 = p[2][2];
	p[1][0] = p10*c - p20*s;
	p[2][0] = p10*s + p20*c;
	p[1][1] = p11*c - p21*s;
	p[2][1] = p11*s + p21*c;
	p[1][2] = p12*c - p22*s;
	p[2][2] = p12*s + p22*c;
	this.update();
}

/** Rotate receiver around y axis of angle a
* @param {float} a angle of rotation
*/
Point3d.prototype.rotY = function(a) {
	c = Math.cos(a);
	s = Math.sin(a);
	var p = this.p;
	var p00 = p[0][0], p01 = p[0][1], p02 = p[0][2];
	var p20 = p[2][0], p21 = p[2][1], p22 = p[2][2];
	var p = this.p;
	p[0][0] = p00*c - p20*s;
	p[2][0] = p00*s + p20*c;
	p[0][1] = p01*c - p21*s;
	p[2][1] = p01*s + p21*c;
	p[0][2] = p02*c - p22*s;
	p[2][2] = p02*s + p22*c;
	this.update();
}

/** Rotate receiver around z axis of angle a
* @param {float} a angle of rotation
*/
Point3d.prototype.rotZ = function(a) {
	c = Math.cos(a);
	s = Math.sin(a);
	var p = this.p;
	var p00 = p[0][0], p01 = p[0][1], p02 = p[0][2];
	var p10 = p[1][0], p11 = p[1][1], p12 = p[1][2];
	p[0][0] = p00*c - p10*s;
	p[1][0] = p00*s + p10*c;
	p[0][1] = p01*c - p11*s;
	p[1][1] = p01*s + p11*c;
	p[0][2] = p02*c - p12*s;
	p[1][2] = p02*s + p12*c;
	this.update();
}

/** Update receiver (change positions of stored copies so as stored shifts between receiver and its copies remain constant
*/
Point3d.prototype.update = function() {
	this.project();
	for (var i=0; i<this.copies.length; i++) {
		this.copies[i].setXYZ(this.xr+this.copiesShifts[i][0],this.yr+this.copiesShifts[i][1],this.zr+this.copiesShifts[i][2]);
		this.copies[i].update();
	}
}

/** Creates new Point3d object, for parameters meaning see {@link Point3d}
* @returns {Point3d} new Point3d object
*/
Point3d.create = function(x,y,z,projection) {
	var ret = new Point3d(x,y,z,projection);
	ret.update();
	return ret;
}




/** Line given by ax+by+c=0 clipped in bounding rectangle
* @class represents line given by equation clipped in bounding rectangle
* @param {Raphael} raphael raphael instance
* @param {float} a a in ax+by+c=0
* @param {float} b b in ax+by+c=0
* @param {float} c c in ax+by+c=0
* @param {Point} lt left top corner of bounding rectangle
* @param {Point} rb right bottom corner of bounding rectangle (lt.x<rb.x && tl.y<rb.y)
* @property {Raphael} raphael Raphael instance
* @property {float} a a in ax+by+c=0
* @property {float} b b in ax+by+c=0
* @property {float} c c in ax+by+c=0
* @property {Point} lt left top corner of bounding rectangle
* @property {Point} rb right bottom corner of bounding rectangle (lt.x<rb.x && tl.y<rb.y)
* @property {Point} point1 1st point of intersection of line with bounding rectangle
* @property {Point} point2 2nd point of intersection of line with bounding rectangle
* @property {Line} line Line object (see {@link Line})
* @property {bool} isOutsideBR if the line is completely outside its bounding rectangle
*/
LineBR = function(raphael,a,b,c,lt,rb) {
	this.raphael = raphael;
	this.a = a==undefined? 1. : a;
	this.b = b==undefined? 1. : b;
	this.c = c==undefined? 1. : c;
	this.lt = lt || Point2d.create();
	this.rb = rb || Point2d.create();
	this.point1 = Point2d.create();
	this.point2 = Point2d.create();
	this.line = Line.create(this.raphael,this.point1,this.point2);
	this.isOutsideBR = true;
}

/** Sets new a of ax+by+c=0
*@param {float} a new a in ax+by+c=0
*/
LineBR.prototype.setABC = function(a,b,c) {
	this.a = a;
	this.b = b;
	this.c = c;
	this.update();
}

/** Sets new a of ax+by+c=0
*@param {float} a new a in ax+by+c=0
*/
LineBR.prototype.setA = function(a) {
	this.a = a;
	this.update();
}

/** Sets new a of ax+by+c=0
*@param {float} a new a in ax+by+c=0
*/
LineBR.prototype.setB = function(b) {
	this.b = b;
	this.update();
}

/** Sets new a of ax+by+c=0
*@param {float} a new a in ax+by+c=0
*/
LineBR.prototype.setC = function(c) {
	this.c = c;
	this.update();
}

/** Hide the line
*/
LineBR.prototype.hide = function() { // TODO
	this.line.obj.hide();
	this.isOutsideBR = true;
}

/** Show the line
*/
LineBR.prototype.show = function() { // TODO
	this.line.obj.show();
	this.isOutsideBR = false;
}

/** Finds sections of given line with given bounding rectangle and sets to this.points
*/
LineBR.prototype.findSections = function() {
	var x1,y1,x2,y2;
	var a = this.a;
	var b = this.b;
	var c = this.c;
	var p1 = this.point1;
	var p2 = this.point2;
	var lt = this.lt;
	var rb = this.rb;
	var ymin = lt.y;
	var ymax = rb.y;
	var xmin = lt.x;
	var xmax = rb.x;
	this.show();
	if (a == 0.) { // horizontal line
		if (-c/b > ymin && -c/b < ymax) { // line is inside BR
			p1.setXY(xmin,-c/b);
			p2.setXY(xmax,-c/b);
		} else { // line is outside BR
			this.hide();
		}
		return;
	}
	if (b == 0.) { // vertical line
		if (-c/a > xmin && -c/a < xmax) { // line is inside BR
			p1.setXY(-c/a,ymin);
			p2.setXY(-c/a,ymax);
		} else { // line is outside BR
			this.hide();
		}
		return;
	}
	// the line is inclined
	x1 = xmin;
	x2 = xmax;
	y1 = -(a*x1+c)/b;
	y2 = -(a*x2+c)/b;
	if (y1<ymin && y2<ymin) { // line is all bellow BR
		this.hide();
		return;
	}
	if (y1>ymax && y2>ymax) { // line is all above BR
		this.hide();
		return;
	}
	if ((y1>ymin && y1<ymax) && (y2>ymin && y2<ymax)) { // line crosses BR from left to right
		p1.setXY(x1,y1);
		p2.setXY(x2,y2);
		return;
	}
	if ((y1<ymin && y2>ymax) || (y1>ymax && y2<ymin)) { // line crosses BR from up to down
		p1.setXY(-(b*ymin+c)/a,ymin);
		p2.setXY(-(b*ymax+c)/a,ymax);
		return;
	}
	if (y1 < ymax && y1 > ymin) { // line crosses left edge of BR
		p1.setXY(x1,y1);
		if (y2 < ymin) { // line crosses top edge of BR
			p2.setXY(-(b*ymin+c)/a,ymin);
		} else { // line crosses bottom edge of BR
			p2.setXY(-(b*ymax+c)/a,ymax);
		}
	} else { // line crosses right edge of BR
		p2.setXY(x2,y2);
		if (y1 < ymin) { // line crosses top edge of BR
			p1.setXY(-(b*ymin+c)/a,ymin);
		} else { // line crosses bottom edge of BR
			p1.setXY(-(b*ymax+c)/a,ymax);
		}
	}
}

/** Normalize receiver values (a,b,c) such that (a,b) is unit vector
*/
LineBR.prototype.normalize = function() {
	if (this.a*this.a + this.b*this.b == 1.) {
		return;
	}
	var n = Math.sqrt(this.a*this.a + this.b*this.b);
	this.a /= n;
	this.b /= n;
	this.c /= n;
}

/** Update receiver
*/
LineBR.prototype.update = function() {
	this.normalize();
	this.line.obj.show();
	this.findSections();
	this.line.update();
}

/** Creates new LineBR object, for parameters meaning see {@link LineBR}
* @returns {LineBR} new LineBR object
*/
LineBR.create = function(raphael,a,b,c,lt,rb) {
	var ret = new LineBR(raphael,a,b,c,lt,rb);
	ret.update();
	return ret;
}





/** Line
* @class represents line connecting two points
* @param {Raphael} raphael raphael instance
* @param {Point} point1 1st Point2d
* @param {Point} point2 2nd Point2d
* @property {Raphael} raphael Raphael instance
* @property {Point} point1 first point of line
* @property {Point} point2 second point of line
* @property {[Lines]} copies array of copies of receiver
* @property {[[Points],[Points]]} copiesPoints array of points of copies of receiver
* @property {Raphael.el} obj pointer to actual raphael object
*/
Line = function(raphael,point1,point2) {
	this.raphael = raphael;
	this.point1 = point1;
	this.point2 = point2;
	this.copies = [];
	this.copiesPoints = [[],[]];
	this.obj = this.raphael? this.raphael.path([]) : null;
}

/** Sets svg path of receiver
*/
Line.prototype.setPathList = function() {
	this.pathList = [["M",this.point1.x,this.point1.y],["L",this.point2.x,this.point2.y]];
}

/** Set this.pathList to this.obj raphael object
*/
Line.prototype.attrObjPath = function() {
	this.obj.attr({"path":this.pathList});
}

/** Update receiver (set this.pathList for given internal variables - x,y,angle... - as well as its copies)
*/
Line.prototype.update = function() {
	this.setPathList();
	this.attrObjPath();
	for (var c=0; c<this.copies.length; c++) {
		this.copies[c].update();
	}
}

/** Copy receiver
* @param {[[Points],[Points]]} points points of to be copied lines
*/
Line.prototype.copy = function(points) { // points=[[point1,point2,point3],[point4,point5,point6]] wll make lines 14,25,36
	for (var n=0; n<points[0].length; n++) {
		this.copiesPoints[0].push(points[0][n]);
		this.copiesPoints[1].push(points[1][n]);
		this.copies.push(this.makeCopy(points[0][n],points[1][n]));
	}
	this.update();
}

/** Returns copy of receiver
* @returns {Line} copy of receiver
*/
Line.prototype.makeCopy = function(point1,point2) {
	var ret = Line.create(this.raphael,point1,point2);
	ret.obj.remove();
	ret.obj = this.obj.clone();
	return ret;
}

/** Creates new Line object, for parameters meaning see {@link Line}
* @returns {Line} new Line object
*/
Line.create = function(raphael,point1,point2) {
	var ret = new Line(raphael,point1,point2);
	ret.update();
	return ret;
}



/** Path (for easier modifying)
* @class represents svg path
* @param {Raphael} raphael raphael instance
* @param {Array} pathList path list for raphael path
* @property {Raphael} raphael raphael instance
* @property {Array} pathList svg array (list) of the object
* @property {Raphael.el} obj actual raphael object
*/
Path = function(raphael,pathList) {
	this.pathList = pathList==undefined? [] : pathList;
	this.raphael = raphael;
	this.obj = this.raphael.path(this.pathList);
}

/** Set this.pathList to this.obj raphael object
*/
Path.prototype.attrObjPath = function() {
	this.obj.attr({"path":this.pathList});
}

/** Sets one specific value in this.pathList (this.pathList[i][j] = val)
* @param {int} i first index
* @param {int} j second index
* @param {float|string} val value to be set
*/
Path.prototype.setItem = function(i,j,val) {
	this.pathList[i][j] = val;
	this.attrObjPath();
}

/** Increment one specific value in this.pathList (this.pathList[i][j] = val)
* @param {int} i first index
* @param {int} j second index
* @param {float} val value of incerementation
*/
Path.prototype.iaddItem = function(i,j,val) {
	this.pathList[i][j] += val;
	this.attrObjPath();
}

/** Sets new svg line path to receiver
* @param {Array} newPathList newpath list
*/
Path.prototype.setNewPath = function(newPathList) {
	this.pathList = newPathList;
	this.attrObjPath();
}

/** Creates new Path object, for parameters meaning see {@link Path}
* @returns {Path} new Path object
*/
Path.create = function(raphael,pathList) {
	var ret = new Path(raphael,pathList);
	return ret;
}






/** Circle (for easier modifying)
* @class represents circle
* @param {Raphael} raphael raphael instance
* @param {float} [x=0.] x coordinate of circle center
* @param {float} [y=0.] y coordinate of circle center
* @param {float} [r=10.] circle radius
* @property {Raphael} raphael raphael instance
* @property {float} x x coordinate of circle center
* @property {float} y y coordinate of circle center
* @property {float} r radius of the circle
* @property {Raphael.circle} obj actual raphael object
* @property {[Circles]} copies array of copies of receiver
* @property {[[float,float]]} copiesShifts array of shifts of copies of receiver
*/
Circle = function(raphael,x,y,r) {
	this.raphael = raphael;
	this.x = x==undefined? 0. : x;
	this.y = y==undefined? 0. : y;
	this.r = r==undefined? 0. : r;
	this.copies = [];
	this.copiesShifts = [];
	this.obj = this.raphael.circle(this.x,this.y,this.r);
}

/** Sets radius of receiver
* @param {float} r new radius
*/
Circle.prototype.setR = function(r) {
	this.r = r;
	this.obj.attr({"r":r});
	this.update();
}

/** Sets new coordinates of receiver
* @param {float} x new x coordinate
* @param {float} y new y coordinate
*/
Circle.prototype.setXY = function(x,y) {
	this.x = x;
	this.y = y;
	this.obj.attr({"cx":x,"cy":y});
}
/** Sets x coordinate of receiver
* @param {float} x new x coordinate
*/
Circle.prototype.setX = function(x) {
	this.x = x;
	this.obj.attr({"cx":x});
}

/** Sets y coordinate of receiver
* @param {float} y new y coordinate
*/
Circle.prototype.setY = function(y) {
	this.y = y;
	this.obj.attr({"cy":y});
}

/** Translate receiver
* @param {float} dx x distance of translation
* @param {float} dy y distance of translation
*/
Circle.prototype.translate = function(dx,dy) {
	this.x += dx;
	this.y += dy;
	this.obj.attr({"cx":this.x,"cy":this.y});
}

/** Translate along x axis of receiver
* @param {float} dx x distance of translation
*/
Circle.prototype.translateX = function(dx) {
	this.translate(dx,0.);
}

/** Translate along y axis of receiver
* @param {float} dy y distance of translation
*/
Circle.prototype.translateY = function(dy) {
	this.translate(0.,dy);
}

/** Copy of receiver
* @param {[[float,float]]} shifts shifts of copies
*/
Circle.prototype.copy = function(shifts) {
	for (var i=0; i<shifts.length; i++) { this.copiesShifts.push(shifts[i]); this.copies.push(this.makeCopy()); this.update()}
}

/** Returns copy of receiver
* @returns {Circle} copy of receiver
*/
Circle.prototype.makeCopy = function() {
	var ret = Circle.create(this.raphael,this.x,this.y,this.r)
	ret.obj.remove();
	ret.obj = this.obj.clone();
	return ret;
}

/** Update
*/
Circle.prototype.update = function() { 
	for (var c=0; c<this.copies.length; c++) {
		this.copies[c].setXY(this.x+this.copiesShifts[c][0],this.y+this.copiesShifts[c][1]);
		this.copies[c].setR(this.r);
	}
}

/** Creates new Circle object, for parameters meaning see {@link Circle}
* @returns {Circle} new Circle object
*/
Circle.create = function(raphael,x,y,r) {
	var ret = new Circle(raphael,x,y,r);
	ret.update();
	return ret;
}





/** Actual slider element - circ
* @param {Raphael} raphael raphael instance
* @param {Object} [attrs={}] parameters passed to raphael.circle.attr
* @returns {Raphael.el} circle object with all methods needed by Slider
*/
SliderCirc = function(raphael,attrs) {
	var attrs = attrs==undefined? {} : attrs;
	var ret = raphael.circle(0,0).attr({"fill":"#f00","r":6,"stroke-width":2,"cursor":"move"});
	ret.attr(attrs);
	/**#nocode+*/
	ret.getX = function() { return this.attr("cx"); }
	ret.setX = function(x) { this.attr({"cx":x}); }
	ret.getY = function() { return this.attr("cy"); }
	ret.setY = function(y) { this.attr({"cy":y}); }
	/**#nocode-*/
	return ret;
}

/** Actual slider element - rect
* @param {Raphael} raphael raphael instance
* @param {Object} [attrs={}] parameters passed to raphael.rect.attr
* @returns {Raphael.el} rect object with all methods needed by Slider
*/
SliderRect = function(raphael,attrs) {
	var attrs = attrs==undefined? {} : attrs;
	var ret = raphael.rect(0,0,13,13,3).attr({"fill":"#f00","cursor":"move"});
	ret.attr(attrs);
	/**#nocode+*/
	ret.getX = function() { return this.attr("x") + .5*this.attr("width"); }
	ret.setX = function(x) { this.attr({"x":x - .5*this.attr("width")}); }
	ret.getY = function() { return this.attr("y") + .5*this.attr("height"); }
	ret.setY = function(y) { this.attr({"y":y - .5*this.attr("height")}); }
	/**#nocode-*/
	return ret;
}




// TODO doc
/** Slider
* @class abstract class epresenting generic slider
* @param {Raphael} raphael raphael instance
* @param {Raphael.el} [slider=SliderCirc] actual slider (must have setX, setX, getY, setY methods)
* @param {Object} params parameters,see below
* @param {float} [params.x=100.] x coordinate of left-top end
* @param {float} [params.y=100.] y coordinate of left-top end
* @param {float} [params.dx=100.] x dimension of slider [pixels]
* @param {float} [params.dy=0.] y dimension of slider [pixels]
* @param {float} [params.val1=0.] value when the slider is at the very beginning
* @param {float} [params.val2=1.] value when the slider is at the very end
* @param {float} [params.val3=val1] extreme beginning value
* @param {float} [params.val4=val2] extreme end value
* @param {float} [params.initVal=(val1+val2)/2] initial value
* @param {float} [params.gap=10.] gap between end of the slider's line and the slider's circle when the circle is at extreme position
* @param {Object} [lineAttrs={}] attrs passed to slider's line
* @property {Raphael} raphael raphael instance
* @property {Raphael.el} slider actual slider (must have setX, setX, getY, setY methods)
* @property {float} x x coordinate of left/upper end
* @property {float} y y coordinate of left/upper end
* @property {float} dx x dimension of slider [pixels]
* @property {float} dy y dimension of slider [pixels]
* @property {float} val1 value when the slider is at the very beginning
* @property {float} val2 value when the slider is at the very end
* @property {float} val3 extreme beginning value
* @property {float} val4 extreme end value
* @property {float} initVal initial value
* @property {float} gap gap between end of the slider's line and the slider's circle when the circle is at extreme position
* @property {float} lenVals length span of getable values
* @property {Raphael.el} line line
*/
// TODO DOC
Slider = function(raphael,slider,params) {
	this.raphael = raphael;
	var params = params==undefined? {} : params;
	this.x = params.x==undefined? 100. : params.x;
	this.y = params.y==undefined? 100. : params.y;
	this.dx = params.dx==undefined? 100. : params.dx;
	this.dy = params.dy==undefined?   0. : params.dy;
	this.len = Math.sqrt(this.dx*this.dx + this.dy*this.dy);
	this.cos = this.dx/this.len;
	this.sin = this.dy/this.len;
	this.lineAttrs = params.lineAttrs==undefined? {} : params.lineAttrs;
	this.gap = params.gap==undefined? 12. : params.gap;
	this.start = this.gap;
	this.stop = this.len - this.start;
	this.val1 = params.val1==undefined? 0. : params.val1;
	this.val2 = params.val2==undefined? 1. : params.val2;
	this.val3 = params.val3==undefined? this.val1 : params.val3;
	this.val4 = params.val4==undefined? this.val2 : params.val4;
	var initVal = params.initVal==undefined? .5*(this.val1+this.val2) : params.initVal;
	this.lenVals = this.len - 2*this.start;
	this.left = this.start + (this.val3-this.val1)/(this.val2-this.val1)*this.lenVals;
	this.right = this.start + (this.val4-this.val1)/(this.val2-this.val1)*this.lenVals;
	this.line = this.raphael.path([["M",this.x,this.y],["l",this.dx,this.dy]]).attr({"stroke":"#000","stroke-width":7})
	this.line.attr(this.lineAttrs)
	this.slider = slider? slider : SliderRect(raphael);
	this.slider.slider = this;
	this.slider.attr({"cursor":"move"})
	/**#nocode+*/
	this.slider.onDragStop = function() {
		this.posX = this.getX();
		this.posY = this.getY();
	}
	this.slider.onDragStop();
	this.setVal(initVal)
	this.minFlag = false;
	this.maxFlag = false;
	/**#nocode-*/
	this.slider.onDragUpdate = function(dx,dy) {
		this.posX += dx;
		this.posY += dy;
		var xLoc = (this.posX-this.slider.x)*this.slider.cos + (this.posY-this.slider.y)*this.slider.sin;
		if (xLoc < this.slider.left) {
			if (!this.slider.minFlag) {
				this.setX(this.slider.x + this.slider.left*this.slider.cos);
				this.setY(this.slider.y + this.slider.left*this.slider.sin);
				this.slider.onmove();
			}
			this.slider.minFlag = true;
			this.slider.maxFlag = false;
		} else if (xLoc > this.slider.right) {
			if (!this.slider.maxFlag) {
				this.setX(this.slider.x + (this.slider.right)*this.slider.cos);
				this.setY(this.slider.y + (this.slider.right)*this.slider.sin);
				this.slider.onmove();
			}
			this.slider.minFlag = false;
			this.slider.maxFlag = true;
		} else {
			this.setX(this.slider.x + xLoc*this.slider.cos);
			this.setY(this.slider.y + xLoc*this.slider.sin);
			this.slider.minFlag = false;
			this.slider.maxFlag = false;
			this.slider.onmove();
		}
		if (this.pos >= this.slider.x+this.slider.start && this.pos <= this.slider.x+this.slider.len-this.slider.start) {
			this.slider.onmove();
		}
	}
	this.slider.drag(onDragMove,onDragStart,onDragStop);
}

/** What to do when moving. Initially empty function, overwrite it by desired actions
*/
Slider.prototype.onmove = function() {}

/** Get current value from receiver
* @returns {float} current value
*/
Slider.prototype.getVal = function() {
	var x = this.slider.getX();
	var y = this.slider.getY();
	var pos = Math.sqrt((x-this.x)*(x-this.x) + (y-this.y)*(y-this.y)) - this.start;
	return this.val1 + pos*(this.val2 - this.val1) / this.lenVals;
}

/** Set new value to receiver
* @param {float} newVal new value
*/
Slider.prototype.setVal = function(newVal) {
	if (newVal < Math.min(this.val3,this.val4)) {
		newVal = Math.min(this.val3,this.val4);
	}
	var newPos = this.start + this.lenVals / (this.val2 - this.val1) * (newVal - this.val1);
	if (newPos < this.start) {
		this.maxFlag = false;
		this.minFlag = true;
		newPos = this.start;
	} else if (newPos > this.stop)  {
		this.maxFlag = true;
		this.minFlag = false;
		newPos = this.stop;
	} else {
		this.maxFlag = this.minFlag = false;
	}
	this.slider.setX(this.x + newPos*this.cos);
	this.slider.setY(this.y + newPos*this.sin);
	this.slider.onDragStop();
}

/** Creates new Slider object, for parameters meaning see {@link Slider}
* @returns {Slider} new Slider object
*/
Slider.create = function(raphael,slider,params) {
	return new Slider(raphael,slider,params);
}



/** Button
* @class represents button
* @param {Raphael} raphael raphael instance
* @param {Object} params parameters, see below
* @param {string} [params.str="str"] displayed string of button
* @param {float} [params.x=100.] x coordinate of left upper end
* @param {float} [params.y=100.] y coordinate of left upper end
* @param {float} [params.width=100.] button width [pixels]
* @param {float} [params.height=30.] button height [pixels]
* @param {float} [params.fontSiye=20.] font siye of button str
* @param {float} [params.rCorner=4.] radius of button corners camber
* @param {color} [params.bgColor="#bbb"] background color of button
* @param {color} [params.bgColorWhenClicked="#aaa"] background color of button when clicked
* @property {Raphael} raphael raphael instance
* @property {float} x x coordinate of left upper end
* @property {float} y y coordinate of left upper end
* @property {float} width length of slider [pixels]
* @property {float} height length of slider [pixels]
* @property {float} fontSize font size of button text
* @property {color} bgColor background color of button
* @property {color} bgColorWhenClicked background color of button when clicked
* @property {Raphael.rect} rect rectangle of receiver
* @property {Raphael.text} text object of receiver
*/
Button = function(raphael,params) {
	this.raphael = raphael;
	var str = params.str==undefined? 'str' : params.str;
	this.x = params.x==undefined? 100. : params.x;
	this.y = params.y==undefined? 100. : params.y;
	this.width = params.width==undefined? 100. : params.width;
	this.height = params.height==undefined? 30. : params.height;
	var fontSize = params.fontSize==undefined? 20. : params.fontSize;
	this.bgColor = params.bgColor==undefined? "#bbb" : params.bgColor;
	this.bgColorWhenClicked = params.bgColorWhenClicked==undefined? "#aaa" : params.bgColorWhenClicked;
	var rCorner = params.rCorner==undefined? 4. : params.rCorner;
	this.rect = this.raphael.rect(this.x,this.y,this.width,this.height,rCorner).attr({"fill":this.bgColor,"cursor":"pointer"})
	this.rect.button = this;
	this.text = this.raphael.text(this.x+.5*this.width,this.y+.5*this.height,str).attr({"font-size":fontSize,"cursor":"pointer"})
	this.text.button = this;
}

/** What to do when clicked. Initially empty function, overwrite it by desired actions
*/
Button.prototype.onclick = function() {}

Button.prototype.doThisWhenClicked = function() {
	this.text.attr({"x":this.x+.5*this.width+2})
	this.rect.attr({"fill":this.bgColorWhenClicked})
	this.onclick();
}

Button.prototype.doThisWhenUnclicked = function() {
	this.text.attr({"x":this.x+.5*this.width})
	this.rect.attr({"fill":this.bgColor})
}

/** Creates new Button object, for parameters meaning see {@link Button}
* @returns {Button} new Button object
*/
Button.create = function(raphael,params) {
	var ret = new Button(raphael,params);
	ret.rect.mousedown(function(event) {this.button.doThisWhenClicked();})
	ret.rect.mouseup(function(event) {this.button.doThisWhenUnclicked();})
	ret.text.mousedown(function(event) {this.button.doThisWhenClicked();})
	ret.text.mouseup(function(event) {this.button.doThisWhenUnclicked();})
	ret.rect.mouseout(function(event) {this.button.doThisWhenUnclicked();})
	ret.text.mouseout(function(event) {this.button.doThisWhenUnclicked();})
	return ret;
}




/** Text in rectangular frame
* @class represents text in rectangular frame
* @param {Raphael} raphael rapheal instance
* @param {Object} params parameters, see below
* @param {float} [params.x=100.] x coordinate of left lower corner [pixel]
* @param {float} [params.y=100.] y coordinate of left lower corner [pixel]
* @param {float} [params.width=100.] width [pixel]
* @param {float} [params.height=100.] height [pixel]
* @param {string} [params.str="string"] string to be shown
* @param {float} [params.fontSize=20.] font size
* @property {float} x x coordinate of top left corner
* @property {float} y y coordinate of top left corner
* @property {float} width width of rectangle
* @property {float} height height of rectangle
* @property {string} str text to display
* @property {float} fontSize font size
* @property {Raphael.text} text raphael text object
* @property {Raphael.rect} rect raphael rect object
*/
TextRect = function(raphael,params) {
	this.raphael = raphael;
	this.x = params.x==undefined? 100. : params.x;
	this.y = params.y==undefined? 100. : params.y;
	this.width = params.width==undefined? 100. : params.width;
	this.height = params.height==undefined? 100. : params.height;
	this.str = params.str==undefined? "string" : params.str;
	this.fontSize = params.fontSize==undefined? 20 : params.fontSize;
	this.rect = raphael.rect(this.x,this.y-this.height,this.width,this.height);
	this.text = raphael.text(this.x+.5*this.width,this.y-.5*this.height,this.str).attr({"font-size":this.fontSize});
}

/** Update receiver (set this.pathList for given internal variables)
*/
TextRect.prototype.update = function() {
	this.rect.attr({"x":this.x,"y":(this.y-this.height)});
	this.text.attr({"x":(this.x+.5*this.width),"y":(this.y-.5*this.height),"text":this.str,"font-size":this.fontSize});
}

/** Sets new x and y position of receiver (this.x = x; this.y = y)
* @param {float} x new x coordinate [pixel]
* @param {float} y new y coordinate [pixel]
*/
TextRect.prototype.setXY = function(x,y) {
	this.x = x;
	this.y = y;
	this.update();
}

/** Sets new x position of receiver (this.x = x)
* @param {float} x new x coordinate [pixel]
*/
TextRect.prototype.setX = function(x) {
	this.x = x;
	this.update();
}

/** Sets new y position of receiver (this.y = y)
* @param {float} y new y coordinate [pixel]
*/
TextRect.prototype.setY = function(y) {
	this.y = y;
	this.update();
}

/** Sets new text of receiver
* @param {string} str new string
*/
TextRect.prototype.setStr = function(str) {
	this.str = str;
	this.update();
}

/** Creates new TextRect object, for parameters meaning see {@link TextRect}
* @returns {TextRect} new TextRect object
*/
TextRect.create = function(raphael,params) {
	var ret = new TextRect(raphael,params);
	ret.update();
	return ret;
}



/** Legend
* @class represents legend
* @param {Raphael} raphael rapheal instance
* @param {[[string,color,float,Object]]} what array of [string,color,width,attrs] arrays. attrs is passed to rapheal.path
* @param {Object} [params={}] parameters of legend
* @param {float} [params.x=0.] x coordinate of left upper corner [pixel]
* @param {float} [params.y=0.] y coordinate of left upper corner [pixel]
* @param {float} [params.len=120.] length of legend lines
* @param {float} [params.fontSize=20.] font size of labels
* @param {float} [params.dy=30.] y difference between individual lines
*/
Legend = function(raphael,what,params) {
	this.x = params.x==undefined? 0. : params.x;
	this.y = params.y==undefined? 0. : params.y;
	this.len = params.len==undefined? 100. : params.len;
	this.fontSize = params.fontSize==undefined? 15 : params.fontSize;
	this.dy = params.dy==undefined? 30 : params.dy;
	var label, color, width, t;
	var y = this.y;
	for (var i=0; i<what.length; i++) {
		label = what[i][0];
		color = what[i].length>1? what[i][1] : "#000";
		width = what[i].length>2? what[i][2] : 1.;
		attrs = what[i].length>3? what[i][3] : {};
		attrs["stroke-width"] = width;
		attrs["stroke"] = color;
		raphael.path([["M",this.x,y],["l",this.len,0]]).attr(attrs);
		raphael.text(this.x+this.len+20,y,label).attr({"font-size":this.fontSize,"text-anchor":"start"});
		y += this.dy
	}
}

/** Creates new Legend object, for parameters meaning see {@link Legend}
* @returns {Legend} new Legend object
*/
Legend.create = function(raphael,what,params) {
	var ret = new Legend(raphael,what,params);
	return ret;
}





/** Rectangle filled with gradient (as browsers does not process rotation so good). Created simply as composition of lines. Works reasonably only in Google Chrome browser.
* @class represents rectangle with linear gradient
* @param {Raphael} raphael rapheal instance
* @param {Object} params parameters, see below
* @param {float} [params.x=100.] x coordinate of left lower corner [pixel]
* @param {float} [params.y=100.] y coordinate of left lower corner [pixel]
* @param {float} [params.width=100.] width [pixel]
* @param {float} [params.height=100.] height [pixel]
* @param {float} [params.angle=0.] angle of rotation along the center of rectangle (!) [deg]
* @param {int} [params.res=TODO] reslution of the gradient
* @param {Array} [params.colors=TODO] colors used in the gradient. The format is [begincolor,EndColor] or [beginColor,[color1,where],[color2,where],...,endColor]. The color format is array normalized [r,g,b], where is relative position of the color. E.g. linear gradient from blue to red [[0,0,1],[1,0,0]] or gradient from blue to red through green in one quarter [[0,0,1],[[0,1,0],0.25],[1,0,0]].
* @property {float} x x coordinate of top left corner
* @property {float} y y coordinate of top left corner
* @property {float} width width of rectangle
* @property {float} height height of rectangle
* @property {float} angle of rotation along the center of rectangle (!)
* @property {int} res rsolution of the gradient
*/
RectGradient = function(raphael,params) {
	var params = params==undefined? {} : params;
	this.raphael = raphael;
	this.x = params.x==undefined? 100. : params.x;
	this.y = params.y==undefined? 100. : params.y;
	this.width = params.width==undefined? 100. : params.width;
	this.height = params.height==undefined? 100. : params.height;
	this.angle = params.angle==undefined? 0. : params.angle;
	this.res = params.res==undefined? this.width : params.res;
	this.colors = params.colors==undefined? [[255,255,255],[255,255,255]] : params.colors;
	this.paths = []
	for (var i=0; i<this.res; i++) {
		this.paths.push(raphael.path().attr({"stroke-width":2.*this.width/this.res}));
	}
}

/** Update receiver
*/
RectGradient.prototype.update = function() {
	var cx = this.x + .5*this.width;
	var cy = this.y + .5*this.height;
	var c =  Math.cos(this.angle*Math.PI/180.);
	var s = -Math.sin(this.angle*Math.PI/180.);
	var x0 = cx + c*(-.5*this.width) - s*(-.5*this.height);
	var y0 = cy + s*(-.5*this.width) + c*(-.5*this.height);
	var dx1 = this.width/this.res*c;
	var dy1 = this.width/this.res*s;
	var dx2 = this.height*(-s);
	var dy2 = this.height*c;
	var goals = [0.];
	var colors = [this.colors[0]];
	for (var i=0; i<this.colors.length-2; i++) {
		goals.push(this.colors[i+1][1]);
		colors.push(this.colors[i+1][0]);
	}
	goals.push(1.);
	colors.push(this.colors[this.colors.length-1]);
	var color, rel, i, j, k;
	for (i=0; i<this.res; i++) {
		rel = i/(this.res - 1);
		j = 0;
		while (goals[j+1] < rel) {
			j += 1;
		}
		color = [];
		for (k=0; k<3; k++) {
			color.push(colors[j][k] + (colors[j+1][k]-colors[j][k])*(rel-goals[j])/(goals[j+1]-goals[j]) );
		}
		this.paths[i].attr({
			"path":[["M", x0+(i+.5)*dx1, y0+(i+.5)*dy1], ["l", dx2, dy2]],
			"stroke":"rgb("+255*color[0]+","+255*color[1]+","+255*color[2]+")"
		});
	}
}

/** Sets new x and y position of receiver (this.x = x; this.y = y)
* @param {float} x new x coordinate [pixel]
* @param {float} y new y coordinate [pixel]
*/
RectGradient.prototype.setXY = function(x,y) {
	this.x = x;
	this.y = y;
	this.update();
}

/** Sets new x position of receiver (this.x = x)
* @param {float} x new x coordinate [pixel]
*/
RectGradient.prototype.setX = function(x) {
	this.x = x;
	this.update();
}

/** Sets new y position of receiver (this.y = y)
* @param {float} y new y coordinate [pixel]
*/
RectGradient.prototype.setY = function(y) {
	this.y = y;
	this.update();
}

/** Sets new angle of receiver (this.angle = angle)
* @param {float} y new y coordinate [pixel]
*/
RectGradient.prototype.setAngle = function(angle) {
	this.angle = angle;
	this.update();
}

/** Sets new colors of receiver
* @param {Array} colors new colors of receiver, see {@link RectGradient}
*/
RectGradient.prototype.setColors = function(colors) {
	this.colors = colors;
	this.update();
}

/** Sets new center of receiver
* @param {float} cx new x coordinate of receiver
* @param {float} cy new y coordinate of receiver
*/
RectGradient.prototype.setCenter = function(cx,cy) {
	this.x = cx - .5*this.width;
	this.y = cy - .5*this.height;
	this.update();
}

/** Creates new RectGradient object, for parameters meaning see {@link RectGradient}
* @returns {RectGradient} new RectGradient object
*/
RectGradient.create = function(raphael,params) {
	var ret = new RectGradient(raphael,params);
	ret.update();
	return ret;
}






/** Check box
* @class represents check box
* @param {Raphael} raphael rapheal instance
* @param {Object} params parameters, see below
* @param {float} [params.x=100.] x coordinate of left upper corner [pixel]
* @param {float} [params.y=100.] y coordinate of left upper corner [pixel]
* @param {float} [params.dim=20.] dimension of the box [pixel]
* @param {bool} [params.isChecked=false] if box is checked
* @param {Object} [params.rectAttrs={}] box (rectangle) attrs passed to this.rect.attr
* @param {Object} [params.pathAttrs={}] path (check) attrs passed to this.path.attr
* @property {float} x x coordinate of top left corner
* @property {float} y y coordinate of top left corner
* @property {float} dim size of rectangle
* @property {bool} isChecked if box is checked
*/
CheckBox = function(raphael,params) {
	var params = params==undefined? {} : params;
	this.raphael = raphael;
	this.x = params.x==undefined? 100. : params.x;
	this.y = params.y==undefined? 100. : params.y;
	this.dim = params.dim==undefined? 20. : params.dim;
	this.isChecked = params.isChecked==undefined? false : params.isChecked;
	var rectAttrs = params.rectAttrs==undefined? {} : params.rectAttrs;
	var pathAttrs = params.pathAttrs==undefined? {} : params.pathAttrs;
	this.rect = this.raphael.rect(this.x,this.y,this.dim,this.dim).attr({"stroke-width":2,"fill":"#fff"}).attr(rectAttrs);
	this.path = this.raphael.path([
		["M",this.x+.2*this.dim,this.y+.4*this.dim],
		["l",.2*this.dim,.3*this.dim],
		["l",.4*this.dim,-.5*this.dim],
	]).attr({"stroke-width":3}).attr(pathAttrs);
	if (!this.isChecked) { this.path.hide(); }
	//
	this.mask = this.raphael.rect(this.x,this.y,this.dim,this.dim).attr({"fill":"#000","opacity":.0,"cursor":"pointer"})
	this.mask.checkBox = this;
	this.mask.click(function() {
		this.checkBox.check(!this.checkBox.isChecked);
		this.checkBox.onchange();
	})
}

/** Update receiver
*/
CheckBox.prototype.update = function() {
}

/** What to do when the box state is changed. Initially empty function
*/
CheckBox.prototype.onchange = function() {}

/** Check (or uncheck) receiver
* @param {bool} [b=true] bool value to be set
*/
CheckBox.prototype.check = function(b) {
	var b = b==undefined? true : b;
	if (b) {
		this.path.show();
		this.isChecked = true;
	} else {
		this.path.hide();
		this.isChecked = false;
	}
}

/** Uncheck receiver
*/
CheckBox.prototype.uncheck = function() {
	this.check(false);
}

/** Creates new CheckBox object, for parameters meaning see {@link CheckBox}
* @returns {CheckBox} new CheckBox object
*/
CheckBox.create = function(raphael,params) {
	var ret = new CheckBox(raphael,params);
	ret.update();
	return ret;
}
